package sun.security.krb5;

import java.security.AccessController;
import sun.security.action.GetBooleanAction;
import java.util.LinkedList;
import sun.security.util.DerInputStream;
import sun.security.util.DerOutputStream;
import java.io.IOException;
import sun.security.krb5.internal.util.KerberosString;
import sun.security.util.DerValue;

public class Realm implements Cloneable
{
    public static final boolean AUTODEDUCEREALM;
    private final String realm;
    
    public Realm(String s) throws RealmException {
        this.realm = parseRealm(s);
    }
    
    public static Realm getDefault() throws RealmException {
        try {
            return new Realm(Config.getInstance().getDefaultRealm());
        }
        catch (RealmException ex) {
            throw ex;
        }
        catch (KrbException ex2) {
            throw new RealmException(ex2);
        }
    }
    
    public Object clone() {
        return this;
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof Realm && this.realm.equals(((Realm)o).realm));
    }
    
    @Override
    public int hashCode() {
        return this.realm.hashCode();
    }
    
    public Realm(DerValue der) throws Asn1Exception, RealmException, IOException {
        if (der == null) {
            throw new IllegalArgumentException("encoding can not be null");
        }
        this.realm = new KerberosString(der).toString();
        if (this.realm == null || this.realm.length() == 0) {
            throw new RealmException(601);
        }
        if (!isValidRealmString(this.realm)) {
            throw new RealmException(600);
        }
        return;
    }
    
    @Override
    public String toString() {
        return this.realm;
    }
    
    public static String parseRealmAtSeparator(String original) throws RealmException {
        if (original == null) {
            throw new IllegalArgumentException("null input name is not allowed");
        }
        String s = new String(original);
        String substring = null;
        int i = 0;
        while (i < s.length()) {
            if (s.charAt(i) == '@' && (i == 0 || s.charAt(i - 1) != '\\')) {
                if (i + 1 < s.length()) {
                    substring = s.substring(i + 1, s.length());
                    break;
                }
                throw new IllegalArgumentException("empty realm part not allowed");
            }
            else {
                ++i;
            }
        }
        if (substring != null) {
            if (substring.length() == 0) {
                throw new RealmException(601);
            }
            if (!isValidRealmString(substring)) {
                throw new RealmException(600);
            }
        }
        return substring;
    }
    
    public static String parseRealmComponent(String original) {
        if (original == null) {
            throw new IllegalArgumentException("null input name is not allowed");
        }
        String s = new String(original);
        String substring = null;
        int i = 0;
        while (i < s.length()) {
            if (s.charAt(i) == '.' && (i == 0 || s.charAt(i - 1) != '\\')) {
                if (i + 1 < s.length()) {
                    substring = s.substring(i + 1, s.length());
                    break;
                }
                break;
            }
            else {
                ++i;
            }
        }
        return substring;
    }
    
    protected static String parseRealm(String s) throws RealmException {
        String realmAtSeparator = parseRealmAtSeparator(s);
        if (realmAtSeparator == null) {
            realmAtSeparator = s;
        }
        if (realmAtSeparator == null || realmAtSeparator.length() == 0) {
            throw new RealmException(601);
        }
        if (!isValidRealmString(realmAtSeparator)) {
            throw new RealmException(600);
        }
        return realmAtSeparator;
    }
    
    protected static boolean isValidRealmString(String s) {
        if (s == null) {
            return false;
        }
        if (s.length() == 0) {
            return false;
        }
        int i = 0;
        while (i < s.length()) {
            if (s.charAt(i) == '/' || s.charAt(i) == '\0') {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    public byte[] asn1Encode() throws Asn1Exception, IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        derOutputStream.putDerValue(new KerberosString(this.realm).toDerValue());
        return derOutputStream.toByteArray();
    }
    
    public static Realm parse(DerInputStream derInputStream, byte b, boolean b2) throws Asn1Exception, IOException, RealmException {
        if (b2 && ((byte)derInputStream.peekByte() & 0x1F) != b) {
            return null;
        }
        DerValue derValue = derInputStream.getDerValue();
        if (b != (derValue.getTag() & 0x1F)) {
            throw new Asn1Exception(906);
        }
        return new Realm(derValue.getData().getDerValue());
    }
    
    public static String[] getRealmsList(String s, String s2) {
        try {
            return parseCapaths(s, s2);
        }
        catch (KrbException ex) {
            return parseHierarchy(s, s2);
        }
    }
    
    private static String[] parseCapaths(String s, String s2) throws KrbException {
        Config instance = Config.getInstance();
        if (!instance.exists(new String[] { "capaths", s, s2 })) {
            throw new KrbException("No conf");
        }
        LinkedList<String> list = new LinkedList();
        while (true) {
            String all = instance.getAll(new String[] { "capaths", s, s2 });
            if (all == null) {
                break;
            }
            String[] split = all.split("\\s+");
            boolean n = false;
            for (int i = split.length - 1; i >= 0; --i) {
                if (!list.contains(split[i]) && !split[i].equals(".") && !split[i].equals(s) && !split[i].equals(s2) && !split[i].equals(s2)) {
                    n = true;
                    list.addFirst(split[i]);
                }
            }
            if (n == 0) {
                break;
            }
            s2 = (String)list.getFirst();
        }
        list.addFirst(s);
        return (String[])list.toArray(new String[list.size()]);
    }
    
    private static String[] parseHierarchy(String s, String s2) {
        String[] split = s.split("\\.");
        String[] split2 = s2.split("\\.");
        int length = split.length;
        int i = split2.length;
        boolean n = false;
        --i;
        --length;
        while (i >= 0 && length >= 0 && split2[i].equals(split[length])) {
            n = true;
            --i;
            --length;
        }
        LinkedList o = new LinkedList();
        for (int j = 0; j <= length; ++j) {
            o.addLast(subStringFrom(split, j));
        }
        if (!(n == 0)) {
            o.addLast(subStringFrom(split, length + 1));
        }
        for (; i >= 0; --i) {
            o.addLast(subStringFrom(split2, i));
        }
        o.removeLast();
        return (String[])o.toArray(new String[o.size()]);
    }
    
    private static String subStringFrom(String[] array, int i) {
        StringBuilder sb = new StringBuilder();
        for (; i < array.length; ++i) {
            if (sb.length() != 0) {
                sb.append('.');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }
    
    static {
        AUTODEDUCEREALM = (boolean)AccessController.doPrivileged(new GetBooleanAction("sun.security.krb5.autodeducerealm"));
    }
}
